//*****************************************************************************
// File Name            : max10_lcd.v
//-----------------------------------------------------------------------------
// Function             : max10 lcd_data gen
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2017/7/25
//*****************************************************************************
module max10_lcd(
    input CLK,
    input RSTB,
	output DCLK,
    output VSD,
    output HSD,
    output [1:0] DATA_R,
    output [1:0] DATA_G,
    output [1:0] DATA_B,
	output DEN,
	input KEY1,
	output LED_0
    );
  //タイング用パラメータ
  parameter hsync_cnt_max = 927;
  parameter hsync_fp = 39;
  parameter hsync_pw = 20;
  parameter hsync_bp = 88;
  parameter vsync_cnt_max = 524;
  parameter vsync_fp = 12;
  parameter vsync_pw = 3;
  parameter vsync_bp = 32;

  wire d_clk;
  reg [1:0] clk_cnt;
  reg tmg_da_en;
  reg [9:0] vsync_cnt;
  reg [10:0] hsync_cnt;
  wire [9:0]  v_p;
  wire [9:0]  h_p;
  reg [7:0] r_data;
  reg [7:0] g_data;
  reg [7:0] b_data;
  reg vsync_ff;
  reg hsync_ff;

//四分周クロック
always @ (posedge CLK or negedge RSTB )
if (RSTB==1'b0) 
  clk_cnt <=2'b00;
else
  clk_cnt <= clk_cnt + 2'b01;
  
assign d_clk = clk_cnt[1];  

//同期カウンタ
always @ (posedge d_clk or negedge RSTB )
if (RSTB==1'b0) 
  begin  
    vsync_cnt <= 10'd0;
    hsync_cnt <= 11'd0;
  end
else
    if (hsync_cnt == hsync_cnt_max)
	  begin
        hsync_cnt <= 11'd0;
	    if (vsync_cnt == vsync_cnt_max)
          vsync_cnt <= 10'd0;
        else
          vsync_cnt <= vsync_cnt + 10'd1;
	  end
    else
	  begin
        hsync_cnt <= hsync_cnt +11'd1;
        vsync_cnt <= vsync_cnt ;
	  end
//タイミング信号生成
always @ (posedge d_clk or negedge RSTB )
if (RSTB==1'b0) 
  begin 
    vsync_ff<= 1'b1;
    hsync_ff <= 1'b1;
    tmg_da_en <= 1'b0;
  end
else
  begin
	//VSYNC
	if ((vsync_cnt==vsync_fp)&&(hsync_cnt==hsync_fp))
	  vsync_ff<= 1'b0;
	else
	  if ((vsync_cnt==(vsync_fp+vsync_pw))&&(hsync_cnt==hsync_fp))
	    vsync_ff<= 1'b1;
	  else
	    vsync_ff<= vsync_ff;
	//HSYNC
	if (hsync_cnt==hsync_fp)
	    hsync_ff <= 1'b0;
   else
	 if  (hsync_cnt==(hsync_fp+hsync_pw))
		 hsync_ff <= 1'b1;
	 else
		 hsync_ff <= hsync_ff;
	//DEN
	if ((vsync_cnt>=(vsync_fp+vsync_bp+1))&&(vsync_cnt <=vsync_cnt_max))
     if (hsync_cnt==(hsync_fp+hsync_bp))
       tmg_da_en <= 1'b1;
	  else
		  if  (hsync_cnt==hsync_cnt_max)
		    tmg_da_en <= 1'b0;
	     else
		    tmg_da_en <= tmg_da_en;
	else
		tmg_da_en <= 1'b0;
  end
//表示画素座標	  
assign v_p =	 ((vsync_cnt>=(vsync_fp+vsync_bp+1))&&(vsync_cnt <=  vsync_cnt_max))?(vsync_cnt-(vsync_fp+vsync_bp+10'd1)):1'b0;	
				  
assign h_p =	 (((vsync_cnt>=(vsync_fp+vsync_bp+1))&&(vsync_cnt <=  vsync_cnt_max))&&
                 ((hsync_cnt>(hsync_fp+hsync_bp))&&(hsync_cnt<=hsync_cnt_max)))?(hsync_cnt-(hsync_fp+hsync_bp+10'd1)):1'b0;	  
//画素データ生成 カラーバー				  
always @ (posedge d_clk or negedge RSTB )
if (RSTB==1'b0) 
  begin
    r_data <=8'h00;  
    g_data <=8'h00;  
    b_data <=8'h00;  
  end
else
  if( tmg_da_en==1'b1)
    if( v_p <10'd240)
      if(h_p<10'h100)//赤
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=8'h00;  
          b_data <=8'h00;
       end			 
      else if(h_p<10'h200)//緑
		  begin
          r_data <=8'h00;  
          g_data <=g_data + 8'h01;  
          b_data <=8'h00;
       end			  
      else if(h_p<10'h300)//青
		  begin
          r_data <=8'h00;  
          g_data <=8'h00;  
          b_data <=b_data + 8'h01;
       end			  
      else 
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=g_data + 8'h01;
          b_data <=b_data + 8'h01;
       end	
    else	
      if(h_p<10'h100)//白
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=g_data + 8'h01;
          b_data <=b_data + 8'h01;
       end			 
      else if(h_p<10'h200)//黄色
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=g_data + 8'h01;  
          b_data <=8'h00;
       end			  
      else if(h_p<10'h300)//水色
		  begin
          r_data <=8'h00;  
          g_data <=g_data + 8'h01;  
          b_data <=b_data + 8'h01;
       end			  
      else 
		  begin
          r_data <=r_data + 8'h01;  
          g_data <=8'h00;  
          b_data <=b_data + 8'h01;
       end		 
  else
    begin
      r_data <=8'h00;  
      g_data <=8'h00;  
      b_data <=8'h00;  
	 end
  //出力信号
  assign DATA_R =(tmg_da_en==1'b1)?r_data[7:6]:2'b00;
  assign DATA_G =(tmg_da_en==1'b1)?g_data[7:6]:2'b00;
  assign DATA_B =(tmg_da_en==1'b1)?b_data[7:6]:2'b00;
  assign HSD = hsync_ff;
  assign VSD = vsync_ff;
  assign DEN = tmg_da_en; 
  assign DCLK = d_clk;
 	  
  assign LED_0 = KEY1;
 
endmodule