//*****************************************************************************
// File Name            : camera_if.v
//-----------------------------------------------------------------------------
// Function             : camera interface
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2017/7/25
//*****************************************************************************
module camera_if (
  pclk,    
  c_vsync, 
  href,    
  data,    
  dclk,
  rstb,
  h_c_en,
  v_c,
  h_c,
  gen_da_en,
  gen_da_r,
  gen_da_g,
  gen_da_b,
  wea,
  addra,
  dina,
  addrb,
  doutb
);

  input pclk;
  input c_vsync;
  input href;
  input [7:0]data;
  input dclk;
  input rstb;
  input h_c_en;
  input [9:0] v_c;
  input [9:0] h_c;
  output gen_da_en;
  output [7:0]gen_da_r;
  output [7:0]gen_da_g;
  output [7:0]gen_da_b;
  output wea;
  output [13 : 0] addra;
  output [15 : 0] dina;
  output [13 : 0] addrb;
  input [15 : 0] doutb;
   
  reg gen_da_en;
  reg [7:0]gen_da_r;
  reg [7:0]gen_da_g;
  reg [7:0]gen_da_b;
  reg c_vsync_d1;
  reg c_vsync_d2;
  reg c_vsync_d3;
  reg href_d1;
  reg href_d2;
  reg href_d3;
  reg [7:0]data_d1; 
  reg [7:0]data_d2; 
  reg [7:0]data_d3; 
  reg  [11:0]i_h_cnt;
  reg  [11:0]i_v_cnt;
  wire [13:0] addra;
  wire [6:0] addra_h_base;
  wire [6:0] addra_h;
  wire [7:0] addra_l_base;
  wire [6:0] addra_l;
  wire       wea;
  wire [15:0] dina;
  wire [4:0]  r_data;
  wire [5:0]  g_data;
  wire [4:0]  b_data;
  // vram signal
  wire [8:0] radr_v_base;
  wire [6:0] radr_v;
  wire [8:0] radr_h_base;
  wire [6:0] radr_h;
  wire [13:0] addrb;
  wire [15:0] doutb;

  //parameter  vram_size_v = 128;         //VRAM垂直方向サイズ
  parameter  vram_size_v = 96;         //VRAM垂直方向サイズ
  parameter  vram_size_h = 128;         //VRAM横方向サイズ
  //parameter  v_start = 12'h60;          //垂直方向切り出し開始位置
  parameter  v_start = 12'd176;          //垂直方向切り出し開始位置
  parameter  v_end = v_start + (vram_size_v-1); //垂直方向切り出し終了位置
  //parameter  h_start = 12'h40;          //水平方向切り出し開始位置
  parameter  h_start = 12'd256;          //水平方向切り出し開始位置
  parameter  h_start_p = h_start * 2;        //水平方向切り出し開始ポイント
  parameter  h_end_p = h_start_p + (2*vram_size_h -1); //水平方向切り出し終了位置
  parameter  out_size_v = vram_size_v * 4;//出力画像垂直方向サイズ
  parameter  out_size_h = vram_size_h * 4;//出力画像水平方向サイズ
  //parameter  out_start_v = 20;          //出力画像垂直方向開始位置
  //parameter  out_start_v = 112;          //出力画像垂直方向開始位置
  parameter  out_start_v = 43;          //出力画像垂直方向開始位置
  //parameter  out_start_h = 500;         //出力画像水平方向開始位置
  //parameter  out_start_h = 272;         //出力画像水平方向開始位置
  parameter  out_start_h = 144;         //出力画像水平方向開始位置
  parameter  out_end_v = out_start_v + out_size_v -1;//出力画像垂直方向終了位置
  parameter  out_end_h = out_start_h + out_size_h -1;//出力画像水平方向終了位置

//カメラデータのりタイミング
always @ (posedge pclk or negedge rstb )//sim
//always @ (posedge clk or negedge rstb )
if (rstb==1'b0)
  begin
    c_vsync_d1 <= 1'b0 ;
    c_vsync_d2 <= 1'b0 ;
    c_vsync_d3 <= 1'b0 ;
    href_d1 <= 1'b0 ;
    href_d2 <= 1'b0 ;
    href_d3 <= 1'b0 ;
    data_d1 <= 8'h0 ; 
    data_d2 <= 8'h0 ; 
    data_d3 <= 8'h0 ; 
  end
else
  begin
    c_vsync_d1 <= c_vsync ;
    c_vsync_d2 <= c_vsync_d1 ;
    c_vsync_d3 <= c_vsync_d2 ;
    href_d1 <= href ;
    href_d2 <= href_d1 ;
    href_d3 <= href_d2 ;
    data_d1 <= data ; 
    data_d2 <= data_d1 ; 
    data_d3 <= data_d2 ; 
  end

//カメラデータの垂直方向カウンタ、水平方向カウンタ
always @ (posedge pclk or negedge rstb )
if (rstb==1'b0)
  begin
     i_v_cnt <= 12'h000;
     i_h_cnt <= 12'h000;
  end
else
  if((c_vsync_d2==1'b1)&&(c_vsync_d3==1'b0)) //vsync
     begin
       i_v_cnt <= 12'h000;
       i_h_cnt <= 12'h000;
     end
  else
     if((href_d2==1'b0)&&(href_d3==1'b1)) //line end
       begin
         i_v_cnt <= i_v_cnt + 12'h001;
         i_h_cnt <= 12'h000;
       end
     else if (href_d2==1'b1) //data enable
       begin
         i_v_cnt <= i_v_cnt ;
         i_h_cnt <= i_h_cnt + 12'h001;
       end
     else
       begin
         i_v_cnt <= i_v_cnt ;
         i_h_cnt <= i_h_cnt ;
       end
   
 //VRAM書き込みアドレス計算
 assign addra_h_base = i_v_cnt - v_start;  
 assign addra_h = ((i_v_cnt>=v_start)&&(i_v_cnt<=v_end))? addra_h_base:7'h00;
 assign addra_l_base = i_h_cnt - h_start_p;  
 assign addra_l = ((i_h_cnt>=h_start_p)&&(i_h_cnt<=h_end_p))?addra_l_base[7:1]:7'h00;
 assign addra = {addra_h,addra_l};
 //assign addra = 14'h3ff-{addra_h,addra_l};

//VRAM書き込みイネーブル
 assign wea =(((i_v_cnt>=v_start)&&(i_v_cnt<=v_end))&&((i_h_cnt>=h_start_p)&&(i_h_cnt<=h_end_p)))? i_h_cnt[0]:1'b0;

//VRAM書き込みデータ

//rgb565->444 for FPGA
//assign r_data = data_d2[7:4]; 
//assign g_data = {data_d2[2:0],data_d1[7]}; 
//assign b_data = data_d1[4:1]; 

//rgb565->444 for FPGA
assign r_data = data_d2[7:3]; 
assign g_data = {data_d2[2:0],data_d1[7:5]}; 
assign b_data = data_d1[4:0]; 

//assign r_data = data_d2[7:4]; 
//assign g_data = {data_d2[2:0],data_d3[7]}; 
//assign b_data = data_d3[4:1]; 

assign dina = {r_data,g_data,b_data};
//画像出力
always @ (posedge dclk or negedge rstb )
if (rstb==1'b0) 
  begin
    gen_da_r <= 8'd0;
    gen_da_g <= 8'd0;
    gen_da_b <= 8'd0;
    gen_da_en <= 1'b0;  end
else
    begin
      gen_da_en <= 1'b0 ;
      gen_da_r <= 8'd0;
      gen_da_g <= 8'd0;
      gen_da_b <= 8'd0;
      // vram 
      if (((v_c>=out_start_v)&&(v_c<=(out_end_v)))&&
          ((h_c>=out_start_h)&&(h_c<=(out_end_h)))&&(h_c_en==1'b1))
        begin
          gen_da_en <= 1'b1;
		  /*
          gen_da_r<={doutb[11:8],4'd0};
          gen_da_g<={doutb[7:4],4'd0};
          gen_da_b<={doutb[3:0],4'd0};
		  */
          gen_da_r<={doutb[15:11],3'd0};
          gen_da_g<={doutb[10:5],2'd0};
          gen_da_b<={doutb[4:0],3'd0};
        end 
      else
        begin
          gen_da_en <= 1'b0;
          gen_da_r<=gen_da_r;
          gen_da_g<=gen_da_g;
          gen_da_b<=gen_da_b;
        end 
    end 
   
//VRAM読み出しアドレス
   //assign radr_v_base = v_c-out_start_v+16;
   assign radr_v_base = v_c-out_start_v;
   //assign radr_v = ((v_c>=out_start_v)&&(v_c<=(out_end_v)))?radr_v_base[7:1]:7'h0;
   assign radr_v = ((v_c>=out_start_v)&&(v_c<=(out_end_v)))?radr_v_base[8:2]:7'h0;
   assign radr_h_base = h_c-out_start_h;
   //assign radr_h = ((h_c>=out_start_h)&&(h_c<=(out_end_h)))?radr_h_base[7:1]:7'h0;
   assign radr_h = ((h_c>=out_start_h)&&(h_c<=(out_end_h)))?radr_h_base[8:2]:7'h0;
   assign addrb ={radr_v,radr_h};

endmodule








